library(data.table)
library(plyr)

timeError <- function(df, groupvars) {
    time <- summarySE(df, measurevar='time', groupvars=groupvars)
    error <- summarySE(df, measurevar='error', groupvars=groupvars)
    data <- data.frame(N=time$N,
                       threshold=time$threshold,
                       numChoices=time$numChoices,
                       increaseRates=time$increaseRates,
                       time=time$time,
                       time.se=time$se,
                       error=error$error,
                       error.se=error$se)
    return(data)
}

timeErrorDiscovery <- function(df, groupvars) {
    time <- summarySE(df, measurevar='time', groupvars=groupvars)
    error <- summarySE(df, measurevar='error', groupvars=groupvars)
    data <- data.frame(N=time$N,
                       threshold=time$threshold,
                       discoveryRate=time$discoveryRate,
                       disruptionRate=time$disruptionRate,
                       recruitmentRate=time$recruitmentRate,
                       discoveryRateOptimal=time$discoveryRateOptimal,
                       time=time$time,
                       time.se=time$se,
                       error=error$error,
                       error.se=error$se)
    return(data)
}

timeErrorVariance <- function(df, groupvars) {
    time <- summarySE(df, measurevar='time', groupvars=groupvars)
    error <- summarySE(df, measurevar='error', groupvars=groupvars)
    data <- data.frame(increaseRates=time$increaseRates,
                       time=time$time,
                       time.se=time$se,
                       error=error$error,
                       error.se=error$se)
    return(data)
}

summarySE <- function(study1=NULL, measurevar, groupvars=NULL, na.rm=FALSE,
                      conf.interval=.95, .drop=TRUE) {
    require(plyr)
    
    # New version of length which can handle NA's: if na.rm==T, don't count them
    length2 <- function (x, na.rm=FALSE) {
        if (na.rm) sum(!is.na(x))
        else       length(x)
    }
    
    # This does the summary. For each group's study1 frame, return a vector with
    # N, mean, and sd
    study1c <- ddply(study1, groupvars, .drop=.drop,
                     .fun = function(xx, col) {
                         c(N    = length2(xx[[col]], na.rm=na.rm),
                           mean = mean   (xx[[col]], na.rm=na.rm),
                           sd   = sd     (xx[[col]], na.rm=na.rm),
                           sum = sum(xx[[col]], na.rm=na.rm)
                         )
                     },
                     measurevar
    )
    
    # Rename the "mean" column    
    study1c <- rename(study1c, c("mean" = measurevar))
    
    study1c$se <- study1c$sd / sqrt(study1c$N)  # Calculate standard error of the mean
    
    # Confidence interval multiplier for standard error
    # Calculate t-statistic for confidence interval: 
    # e.g., if conf.interval is .95, use .975 (above/below), and use df=N-1
    ciMult <- qt(conf.interval/2 + .5, study1c$N-1)
    study1c$ci <- study1c$se * ciMult
    
    return(study1c)
}

summarizeData <- function(fileName) {
    df <- fread(paste0(inputFolder, fileName, '.csv'))
    df$error <- 1 - df$option1 # Note: this may be calculated already on new simulations; drop if error!
    # groupvars <- if(fileName=='numTypes') c('threshold', 'numChoices') else c('threshold', 'increaseRates')
    groupvars <- c('threshold', 'numChoices', 'increaseRates')
    if (fileName=='optimalPayoff') {
        df$increaseRates <- ifelse(df$increaseRates=="((2,), (1,))", "<2,1>", 
                                   ifelse(df$increaseRates=="((3,), (1,))", "<3,1>", "<4,1>"))
    }
    if (fileName=='suboptimalPayoff') {
        df$increaseRates <- ifelse(df$increaseRates=="((4,), (1,))", "<4,1>", 
                                   ifelse(df$increaseRates=="((4,), (2,))", "<4,2>", "<4,3>"))
    }
    if (fileName=='noise') {
        df$increaseRates <- ifelse(df$increaseRates=="((2, 2, 2, 2), (1, 1, 1, 1))", "No Noise", "Noise")
    }
    output <- timeError(df, groupvars=groupvars)
    write.csv(output, file=paste0(outputFolder, fileName, '.csv'), row.names=FALSE)
    rm(list=ls())
    gc()
}

summarizeDiscovery <- function(fileName) {
    df <- fread(paste0(inputFolder, fileName, '.csv'))
    groupvars <- c('threshold', 'discoveryRate', 'disruptionRate', 'recruitmentRate','discoveryRateOptimal')

    output <- timeErrorDiscovery(df, groupvars=groupvars)
    write.csv(output, file=paste0(outputFolder, fileName, '.csv'), row.names=FALSE)
    rm(list=ls())
    gc()
}

summarizeVariance <- function(fileName) {
    df <- fread(paste0(inputFolder, fileName, '.csv'))
    output <- timeErrorVariance(df, groupvars='increaseRates')
    output$highPayoff <- with(output,
                              ifelse(increaseRates == "((2, 2), (2, 2))", 2,
                              ifelse(increaseRates == "((1, 3), (2, 2))", 3,
                              ifelse(increaseRates == "((1, 1, 4), (2, 2, 2))", 4,
                              ifelse(increaseRates == "((1, 1, 1, 5), (2, 2, 2, 2))", 5,
                              ifelse(increaseRates == "((1, 1, 1, 1, 6), (2, 2, 2, 2, 2))", 6,
                              ifelse(increaseRates == "((1, 1, 1, 1, 1, 7), (2, 2, 2, 2, 2, 2))", 7,
                              ifelse(increaseRates == "((1, 1, 1, 1, 1, 1, 8), (2, 2, 2, 2, 2, 2, 2))", 8,
                              ifelse(increaseRates == "((1, 1, 1, 1, 1, 1, 1, 9), (2, 2, 2, 2, 2, 2, 2, 2))", 9,
                              ifelse(increaseRates == "((1, 1, 1, 1, 1, 1, 1, 1, 10), (2, 2, 2, 2, 2, 2, 2, 2, 2))", 10, 'error'))))))))))
    output <- data.frame(highPayoff = output$highPayoff,
                         safeChoice = output$error,
                         time = output$time)
    write.csv(output, file=paste0(outputFolder, fileName, '.csv'), row.names=FALSE)
}

inputFolder <- "C:/Users/David/git/johnBees/Bees and Urns/"
outputFolder <- "D:/Dropbox/Research/John/Bees/beePaper/data/"
memory.limit(46000)


### Create CSV for Figure 1 -- Number of Options###
summarizeData(fileName='numOptions')

### Create CSV for Figure 2 -- Varying Optimal Payoff ###
summarizeData(fileName='optimalPayoff')

### Create CSV for Figure 3 -- Varying Suboptimal Payoff ###
summarizeData(fileName='suboptimalPayoff')

### Create CSV for Figure 4 -- Noise ###
summarizeData(fileName='noise')

### Creating CSV for Figure 6 ###
summarizeData(fileName='risk')

### Creating CSV for Figure 8 -- Discovery ###
summarizeDiscovery(fileName="discovery")

### Creating CSV for new Variance Analysis
summarizeVariance(fileName="variance")
